<?php

namespace Drupal\configure_contextual_links\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\ContextualLinkManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the Configure Contextual Links settings form.
 */
class ContextualLinkSettingsForm extends ConfigFormBase {

  /**
   * A contextual link plugin manager to deal with contextual links.
   *
   * @var \Drupal\Core\Menu\ContextualLinkManager
   */
  protected $contextualLinkManager;

  /**
   * Constructs a \Drupal\configure_contextual_links\SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Menu\ContextualLinkManager $contextual_link_manager
   *   A contextual link plugin manager to deal with contextual links.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ContextualLinkManager $contextual_link_manager) {
    parent::__construct($config_factory);
    $this->contextualLinkManager = $contextual_link_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.menu.contextual_link'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'configure_contextual_links_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['configure_contextual_links.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    \Drupal::service('messenger')->addMessage($this->t(
      'Contextual links get stored in browser session data. You may need to 
      restart your browser or clear your session data to see this configuration 
      take effect.'
    ));

    $config = $this->config('configure_contextual_links.settings');
    $form = parent::buildForm($form, $form_state);

    $definitions = $this->contextualLinkManager->getDefinitions();

    $options = [];

    foreach ($definitions as $definition) {
      $options[$definition['id']] = $definition['group'] . ': ' . $definition['title'];
    }

    $form['disabled'] = [
      '#title' => $this->t('Disable Contextual Links'),
      '#type' => 'checkboxes',
      '#options' => $options,
      '#required' => FALSE,
      '#default_value' => $config->get('disabled') ?? [],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('configure_contextual_links.settings');
    $disabled = $form_state->getValue('disabled');
    $clean_disabled = [];
    $disabled_ids = array_keys($disabled);
    foreach ($disabled_ids as $disabled_id) {
      if ($disabled[$disabled_id]) {
        $clean_disabled[] = $disabled_id;
      }
    }
    $config->set('disabled', $clean_disabled);
    $config->save();
    $this->contextualLinkManager->clearCachedDefinitions();
    parent::submitForm($form, $form_state);
  }

}
